@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2009 Nick Schermer <nick@xfce.org>
+ * Copyright (C) 2009-2010 Nick Schermer <nick@xfce.org>
  *
- * This program is free software; you can redistribute it and/or modify it
+ * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
  * Software Foundation; either version 2 of the License, or (at your option)
  * any later version.
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
+ * This library is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -431,7 +431,7 @@ actions_plugin_button_clicked (GtkWidget     *button,
                                ActionsPlugin *plugin)
 {
   ActionType    action;
-  XfceSMClient *sm_client;
+  XfceSMClient *sm_client = NULL;
 
   panel_return_if_fail (XFCE_IS_ACTIONS_PLUGIN (plugin));
 
@@ -441,10 +441,13 @@ actions_plugin_button_clicked (GtkWidget     *button,
   else
     action = plugin->second_action;
 
+#if 0
   /* get the active session */
+  /* TODO this is not implemented in XfceSMClient */
   sm_client = xfce_sm_client_get ();
   if (!xfce_sm_client_is_connected (sm_client))
     sm_client = NULL;
+#endif
 
   switch (action)
     {
