@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2009 Nick Schermer <nick@xfce.org>
+ * Copyright (C) 2008-2010 Nick Schermer <nick@xfce.org>
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -40,6 +40,7 @@
 
 #define ARROW_BUTTON_SIZE              (12)
 #define TOOLTIP_ICON_SIZE              (32)
+#define MENU_ICON_SIZE                 (32)
 #define MENU_POPUP_DELAY               (225)
 #define NO_ARROW_INSIDE_BUTTON(plugin) ((plugin)->arrow_position != LAUNCHER_ARROW_INTERNAL \
                                         || LIST_HAS_ONE_OR_NO_ENTRIES ((plugin)->items))
@@ -62,6 +63,7 @@ static void       launcher_plugin_removed                       (XfcePanelPlugin
 static gboolean   launcher_plugin_remote_event                  (XfcePanelPlugin      *panel_plugin,
                                                                  const gchar          *name,
                                                                  const GValue         *value);
+static void       launcher_plugin_save_delayed                  (LauncherPlugin       *plugin);
 static void       launcher_plugin_save                          (XfcePanelPlugin      *panel_plugin);
 static void       launcher_plugin_orientation_changed           (XfcePanelPlugin      *panel_plugin,
                                                                  GtkOrientation        orientation);
@@ -121,6 +123,12 @@ static gboolean   launcher_plugin_button_drag_motion            (GtkWidget
                                                                  gint                  y,
                                                                  guint                 drag_time,
                                                                  LauncherPlugin       *plugin);
+static gboolean   launcher_plugin_button_drag_drop              (GtkWidget            *widget,
+                                                                 GdkDragContext       *context,
+                                                                 gint                  x,
+                                                                 gint                  y,
+                                                                 guint                 drag_time,
+                                                                 LauncherPlugin       *plugin);
 static void       launcher_plugin_button_drag_leave             (GtkWidget            *widget,
                                                                  GdkDragContext       *context,
                                                                  guint                 drag_time,
@@ -200,6 +208,8 @@ struct _LauncherPlugin
 
   GFile             *config_directory;
   GFileMonitor      *config_monitor;
+
+  guint              save_timeout_id;
 };
 
 enum
@@ -314,7 +324,9 @@ launcher_plugin_class_init (LauncherPluginClass *klass)
 
   launcher_menu_icon_size = gtk_icon_size_from_name ("panel-launcher-menu");
   if (launcher_menu_icon_size == GTK_ICON_SIZE_INVALID)
-    launcher_menu_icon_size = gtk_icon_size_register ("panel-launcher-menu", 32, 32);
+    launcher_menu_icon_size = gtk_icon_size_register ("panel-launcher-menu",
+                                                      MENU_ICON_SIZE,
+                                                      MENU_ICON_SIZE);
 
   launcher_tooltip_icon_size = gtk_icon_size_from_name ("panel-launcher-tooltip");
   if (launcher_tooltip_icon_size == GTK_ICON_SIZE_INVALID)
@@ -339,6 +351,7 @@ launcher_plugin_init (LauncherPlugin *plugin)
   plugin->child = NULL;
   plugin->tooltip_cache = NULL;
   plugin->menu_timeout_id = 0;
+  plugin->save_timeout_id = 0;
 
   /* monitor the default icon theme for changes */
   icon_theme = gtk_icon_theme_get_default ();
@@ -354,8 +367,6 @@ launcher_plugin_init (LauncherPlugin *plugin)
   xfce_panel_plugin_add_action_widget (XFCE_PANEL_PLUGIN (plugin), plugin->button);
   gtk_widget_set_has_tooltip (plugin->button, TRUE);
   gtk_widget_set_name (plugin->button, "launcher-button");
-  gtk_drag_dest_set (plugin->button, GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
-                     drop_targets, G_N_ELEMENTS (drop_targets), GDK_ACTION_COPY);
   g_signal_connect (G_OBJECT (plugin->button), "button-press-event",
       G_CALLBACK (launcher_plugin_button_press_event), plugin);
   g_signal_connect (G_OBJECT (plugin->button), "button-release-event",
@@ -366,11 +377,15 @@ launcher_plugin_init (LauncherPlugin *plugin)
       G_CALLBACK (launcher_plugin_button_drag_data_received), plugin);
   g_signal_connect (G_OBJECT (plugin->button), "drag-motion",
       G_CALLBACK (launcher_plugin_button_drag_motion), plugin);
+  g_signal_connect (G_OBJECT (plugin->button), "drag-drop",
+      G_CALLBACK (launcher_plugin_button_drag_drop), plugin);
   g_signal_connect (G_OBJECT (plugin->button), "drag-leave",
       G_CALLBACK (launcher_plugin_button_drag_leave), plugin);
   g_signal_connect_after (G_OBJECT (plugin->button), "expose-event",
       G_CALLBACK (launcher_plugin_button_expose_event), plugin);
 
+
+
   plugin->child = xfce_panel_image_new ();
   gtk_container_add (GTK_CONTAINER (plugin->button), plugin->child);
 
@@ -379,16 +394,20 @@ launcher_plugin_init (LauncherPlugin *plugin)
   xfce_panel_plugin_add_action_widget (XFCE_PANEL_PLUGIN (plugin), plugin->arrow);
   gtk_button_set_relief (GTK_BUTTON (plugin->arrow), GTK_RELIEF_NONE);
   gtk_widget_set_name (plugin->button, "launcher-arrow");
-  gtk_drag_dest_set (plugin->arrow, GTK_DEST_DEFAULT_MOTION,
-                     drop_targets, G_N_ELEMENTS (drop_targets),
-                     GDK_ACTION_COPY);
   g_signal_connect (G_OBJECT (plugin->arrow), "button-press-event",
       G_CALLBACK (launcher_plugin_arrow_press_event), plugin);
   g_signal_connect (G_OBJECT (plugin->arrow), "drag-motion",
       G_CALLBACK (launcher_plugin_arrow_drag_motion), plugin);
+  g_signal_connect (G_OBJECT (plugin->button), "drag-drop",
+      G_CALLBACK (launcher_plugin_button_drag_drop), plugin);
   g_signal_connect (G_OBJECT (plugin->arrow), "drag-leave",
       G_CALLBACK (launcher_plugin_arrow_drag_leave), plugin);
 
+  /* accept all sorts of drag data, but filter in drag-drop, so we can
+   * send other sorts of drops to parent widgets */
+  gtk_drag_dest_set (plugin->button, 0, NULL, 0, 0);
+  gtk_drag_dest_set (plugin->arrow, 0, NULL, 0, 0);
+
   /* sync button states */
   g_signal_connect (G_OBJECT (plugin->button), "state-changed",
       G_CALLBACK (launcher_plugin_button_state_changed), plugin->arrow);
@@ -483,9 +502,9 @@ launcher_plugin_item_changed (GarconMenuItem *item,
 
 
 static gboolean
-launcher_plugin_item_dupplicate (GFile   *src_file,
-                                 GFile   *dst_file,
-                                 GError **error)
+launcher_plugin_item_duplicate (GFile   *src_file,
+                                GFile   *dst_file,
+                                GError **error)
 {
   GKeyFile *key_file;
   gchar    *contents = NULL;
@@ -506,7 +525,7 @@ launcher_plugin_item_dupplicate (GFile   *src_file,
   if (!g_key_file_load_from_data (key_file, contents, length, 0, error))
     goto err1;
 
-  /* store the source uri in the desktop file for resore purposes */
+  /* store the source uri in the desktop file for restore purposes */
   uri = g_file_get_uri (src_file);
   g_key_file_set_string (key_file, G_KEY_FILE_DESKTOP_GROUP, "X-XFCE-Source", uri);
   g_free (uri);
@@ -535,7 +554,8 @@ err1:
 static GarconMenuItem *
 launcher_plugin_item_load (LauncherPlugin *plugin,
                            const gchar    *str,
-                           gboolean       *desktop_id_return)
+                           gboolean       *desktop_id_return,
+                           gboolean       *location_changed)
 {
   GFile          *src_file, *dst_file;
   gchar          *src_path, *dst_path;
@@ -560,17 +580,20 @@ launcher_plugin_item_load (LauncherPlugin *plugin,
           dst_path = launcher_plugin_unique_filename (plugin);
           dst_file = g_file_new_for_path (dst_path);
 
-          /* create a dupplicate in the config directory */
-          if (launcher_plugin_item_dupplicate (src_file, dst_file, &error))
+          /* create a duplicate in the config directory */
+          if (launcher_plugin_item_duplicate (src_file, dst_file, &error))
             {
               /* use the new file */
               g_object_unref (G_OBJECT (src_file));
               src_file = dst_file;
+
+              if (G_LIKELY (location_changed != NULL))
+                *location_changed = TRUE;
             }
           else
             {
               src_path = g_file_get_parse_name (src_file);
-              g_warning ("Failed to create dupplicate of desktop file \"%s\" "
+              g_warning ("Failed to create duplicate of desktop file \"%s\" "
                           "to \"%s\": %s", src_path, dst_path, error->message);
               g_error_free (error);
               g_free (src_path);
@@ -578,7 +601,7 @@ launcher_plugin_item_load (LauncherPlugin *plugin,
               /* continue using the source file, the user won't be able to
                * edit the item, but atleast we have something that works in
                * the panel */
-              g_object_unref (G_OBJECT (dst_path));
+              g_object_unref (G_OBJECT (dst_file));
             }
 
           g_free (dst_path);
@@ -633,13 +656,43 @@ launcher_plugin_item_load (LauncherPlugin *plugin,
 
 
 static void
-launcher_plugin_items_free (GSList *items)
+launcher_plugin_items_delete_configs (LauncherPlugin *plugin)
 {
-  if (items != NULL)
+  GSList   *li;
+  GFile    *file;
+  gboolean  succeed = TRUE;
+  GError   *error = NULL;
+
+  panel_return_if_fail (G_IS_FILE (plugin->config_directory));
+
+  /* cleanup desktop files in the config dir */
+  for (li = plugin->items; succeed && li != NULL; li = li->next)
     {
-      g_slist_foreach (items, (GFunc) g_object_unref, NULL);
-      g_slist_free (items);
-      items = NULL;
+      file = garcon_menu_item_get_file (li->data);
+      if (g_file_has_prefix (file, plugin->config_directory))
+        succeed = g_file_delete (file, NULL, &error);
+      g_object_unref (G_OBJECT (file));
+    }
+
+  if (!succeed)
+    {
+      g_message ("launcher-%d: Failed to cleanup the configuration: %s",
+                 xfce_panel_plugin_get_unique_id (XFCE_PANEL_PLUGIN (plugin)),
+                 error->message);
+      g_error_free (error);
+    }
+}
+
+
+
+static void
+launcher_plugin_items_free (LauncherPlugin *plugin)
+{
+  if (G_LIKELY (plugin->items != NULL))
+    {
+      g_slist_foreach (plugin->items, (GFunc) g_object_unref, NULL);
+      g_slist_free (plugin->items);
+      plugin->items = NULL;
     }
 }
 
@@ -658,6 +711,8 @@ launcher_plugin_items_load (LauncherPlugin *plugin,
   GHashTable     *pool = NULL;
   gboolean        desktop_id;
   gchar          *uri;
+  gboolean        items_modified = FALSE;
+  gboolean        location_changed;
 
   panel_return_if_fail (XFCE_IS_LAUNCHER_PLUGIN (plugin));
   panel_return_if_fail (array != NULL);
@@ -674,7 +729,8 @@ launcher_plugin_items_load (LauncherPlugin *plugin,
 
       /* try to load the item */
       desktop_id = FALSE;
-      item = launcher_plugin_item_load (plugin, str, &desktop_id);
+      location_changed = FALSE;
+      item = launcher_plugin_item_load (plugin, str, &desktop_id, &location_changed);
       if (G_LIKELY (item == NULL))
         {
           /* str did not look like a desktop-id, so no need to look
@@ -682,6 +738,11 @@ launcher_plugin_items_load (LauncherPlugin *plugin,
           if (!desktop_id)
             continue;
 
+          /* we are going to load an desktop_id from the item pool,
+           * even if this failes, save the new item list, so we don't
+           * try this again in the future */
+          items_modified = TRUE;
+
           /* load the pool with desktop items */
           if (pool == NULL)
             pool = launcher_plugin_garcon_menu_pool ();
@@ -692,7 +753,7 @@ launcher_plugin_items_load (LauncherPlugin *plugin,
             {
               /* we want an editable file, so try to make a copy */
               uri = garcon_menu_item_get_uri (pool_item);
-              item = launcher_plugin_item_load (plugin, uri, NULL);
+              item = launcher_plugin_item_load (plugin, uri, NULL, NULL);
               g_free (uri);
 
               /* if something failed, use the pool item, but this one
@@ -705,6 +766,10 @@ launcher_plugin_items_load (LauncherPlugin *plugin,
           if (item == NULL)
             continue;
         }
+      else if (location_changed)
+        {
+          items_modified = TRUE;
+        }
 
       /* add the item to the list */
       panel_assert (GARCON_IS_MENU_ITEM (item));
@@ -716,9 +781,16 @@ launcher_plugin_items_load (LauncherPlugin *plugin,
   if (G_UNLIKELY (pool != NULL))
     g_hash_table_destroy (pool);
 
+  /* remove config files of items not in the new config */
+  launcher_plugin_items_delete_configs (plugin);
+
   /* release the old menu items and set new one */
-  launcher_plugin_items_free (plugin->items);
+  launcher_plugin_items_free (plugin);
   plugin->items = items;
+
+  /* store the new item list */
+  if (items_modified)
+    launcher_plugin_save_delayed (plugin);
 }
 
 
@@ -743,9 +815,14 @@ launcher_plugin_set_property (GObject      *object,
       /* load new items from the array */
       array = g_value_get_boxed (value);
       if (G_LIKELY (array != NULL))
-        launcher_plugin_items_load (plugin, array);
+        {
+          launcher_plugin_items_load (plugin, array);
+        }
       else
-        launcher_plugin_items_free (plugin->items);
+        {
+          launcher_plugin_items_delete_configs (plugin);
+          launcher_plugin_items_free (plugin);
+        }
 
       /* emit signal */
       g_signal_emit (G_OBJECT (plugin), launcher_signals[ITEMS_CHANGED], 0);
@@ -894,7 +971,7 @@ launcher_plugin_file_changed (GFileMonitor      *monitor,
       launcher_plugin_menu_destroy (plugin);
 
       /* save the new config */
-      g_object_notify (G_OBJECT (plugin), "items");
+      launcher_plugin_save_delayed (plugin);
 
       /* update the dialog */
       g_signal_emit (G_OBJECT (plugin), launcher_signals[ITEMS_CHANGED], 0);
@@ -1004,14 +1081,16 @@ launcher_plugin_free_data (XfcePanelPlugin *panel_plugin)
       g_object_unref (G_OBJECT (plugin->config_monitor));
     }
 
-  if (plugin->config_directory != NULL)
-    g_object_unref (G_OBJECT (plugin->config_directory));
+  if (plugin->save_timeout_id != 0)
+    g_source_remove (plugin->save_timeout_id);
 
   /* destroy the menu and timeout */
   launcher_plugin_menu_destroy (plugin);
 
-  /* free items */
-  launcher_plugin_items_free (plugin->items);
+  launcher_plugin_items_free (plugin);
+
+  if (plugin->config_directory != NULL)
+    g_object_unref (G_OBJECT (plugin->config_directory));
 
   /* stop watching the icon theme */
   if (plugin->theme_change_id != 0)
@@ -1032,9 +1111,6 @@ launcher_plugin_removed (XfcePanelPlugin *panel_plugin)
 {
   LauncherPlugin *plugin = XFCE_LAUNCHER_PLUGIN (panel_plugin);
   GError         *error = NULL;
-  GSList         *li;
-  GFile          *item_file;
-  gboolean        result = TRUE;
 
   panel_return_if_fail (G_IS_FILE (plugin->config_directory));
 
@@ -1051,19 +1127,9 @@ launcher_plugin_removed (XfcePanelPlugin *panel_plugin)
     }
 
   /* cleanup desktop files in the config dir */
-  for (li = plugin->items; result && li != NULL; li = li->next)
-    {
-      item_file = garcon_menu_item_get_file (li->data);
-      if (g_file_has_prefix (item_file, plugin->config_directory))
-        result = g_file_delete (item_file, NULL, &error);
-      g_object_unref (G_OBJECT (item_file));
-    }
+  launcher_plugin_items_delete_configs (plugin);
 
-  /* remove config dir if everything went fine */
-  if (result)
-    result = g_file_delete (plugin->config_directory, NULL, &error);
-
-  if (!result)
+  if (!g_file_delete (plugin->config_directory, NULL, &error))
     {
       g_message ("launcher-%d: Failed to cleanup the configuration: %s",
                  xfce_panel_plugin_get_unique_id (panel_plugin),
@@ -1106,6 +1172,37 @@ launcher_plugin_remote_event (XfcePanelPlugin *panel_plugin,
 
 
 
+static void
+launcher_plugin_save_delayed_timeout_destroyed (gpointer user_data)
+{
+  XFCE_LAUNCHER_PLUGIN (user_data)->save_timeout_id = 0;
+}
+
+
+
+static gboolean
+launcher_plugin_save_delayed_timeout (gpointer user_data)
+{
+  launcher_plugin_save (XFCE_PANEL_PLUGIN (user_data));
+
+  return FALSE;
+}
+
+
+
+static void
+launcher_plugin_save_delayed (LauncherPlugin *plugin)
+{
+  if (plugin->save_timeout_id != 0)
+    g_source_remove (plugin->save_timeout_id);
+
+  plugin->save_timeout_id = g_timeout_add_seconds_full (G_PRIORITY_LOW, 1,
+      launcher_plugin_save_delayed_timeout, plugin,
+      launcher_plugin_save_delayed_timeout_destroyed);
+}
+
+
+
 static void
 launcher_plugin_save (XfcePanelPlugin *panel_plugin)
 {
@@ -1214,6 +1311,9 @@ launcher_plugin_screen_position_changed (XfcePanelPlugin    *panel_plugin,
   /* set the new arrow direction */
   xfce_arrow_button_set_arrow_type (XFCE_ARROW_BUTTON (plugin->arrow),
       xfce_panel_plugin_arrow_type (panel_plugin));
+
+  /* destroy the menu to update sort order */
+  launcher_plugin_menu_destroy (plugin);
 }
 
 
@@ -1419,6 +1519,7 @@ launcher_plugin_menu_construct (LauncherPlugin *plugin)
   GtkWidget      *mi, *image;
   const gchar    *name, *icon_name;
   GSList         *li;
+  gint            w, h, size;
 
   panel_return_if_fail (XFCE_IS_LAUNCHER_PLUGIN (plugin));
   panel_return_if_fail (plugin->menu == NULL);
@@ -1432,6 +1533,12 @@ launcher_plugin_menu_construct (LauncherPlugin *plugin)
   /* get the arrow type of the plugin */
   arrow_type = xfce_arrow_button_get_arrow_type (XFCE_ARROW_BUTTON (plugin->arrow));
 
+  /* size of the menu items */
+  if (gtk_icon_size_lookup (launcher_menu_icon_size, &w, &h))
+    size = MIN (w, h);
+  else
+    size = MENU_ICON_SIZE;
+
   /* walk through the menu entries */
   for (li = plugin->items, n = 0; li != NULL; li = li->next, n++)
     {
@@ -1466,16 +1573,17 @@ launcher_plugin_menu_construct (LauncherPlugin *plugin)
         }
 
       /* depending on the menu position we prepend or append */
-      if (G_UNLIKELY (arrow_type == GTK_ARROW_DOWN))
-        gtk_menu_shell_append (GTK_MENU_SHELL (plugin->menu), mi);
-      else
+      if (G_UNLIKELY (arrow_type == GTK_ARROW_UP))
         gtk_menu_shell_prepend (GTK_MENU_SHELL (plugin->menu), mi);
+      else
+        gtk_menu_shell_append (GTK_MENU_SHELL (plugin->menu), mi);
 
       /* set the icon if one is set */
       icon_name = garcon_menu_item_get_icon_name (item);
       if (!exo_str_is_empty (icon_name))
         {
-          image = gtk_image_new_from_icon_name (icon_name, launcher_menu_icon_size);
+          image = xfce_panel_image_new_from_source (icon_name);
+          xfce_panel_image_set_size (XFCE_PANEL_IMAGE (image), size);
           gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
           gtk_widget_show (image);
         }
@@ -1775,6 +1883,34 @@ launcher_plugin_button_drag_data_received (GtkWidget        *widget,
 
 
 
+static GdkAtom
+launcher_plugin_supported_drop (GdkDragContext *context,
+                                GtkWidget      *widget)
+{
+  GList           *li;
+  GdkAtom          target;
+  guint            i;
+  GdkModifierType  modifiers = 0;
+
+  /* do not handle drops if control is pressed */
+  gdk_window_get_pointer (gtk_widget_get_window (widget), NULL, NULL, &modifiers);
+  if (PANEL_HAS_FLAG (modifiers, GDK_CONTROL_MASK))
+    return GDK_NONE;
+
+  /* check if we support the target */
+  for (li = context->targets; li; li = li->next)
+    {
+      target = GDK_POINTER_TO_ATOM (li->data);
+      for (i = 0; i < G_N_ELEMENTS (drop_targets); i++)
+        if (target == gdk_atom_intern_static_string (drop_targets[i].target))
+          return target;
+    }
+
+  return GDK_NONE;
+}
+
+
+
 static gboolean
 launcher_plugin_button_drag_motion (GtkWidget      *widget,
                                     GdkDragContext *context,
@@ -1785,6 +1921,9 @@ launcher_plugin_button_drag_motion (GtkWidget      *widget,
 {
   panel_return_val_if_fail (XFCE_IS_LAUNCHER_PLUGIN (plugin), FALSE);
 
+  if (launcher_plugin_supported_drop (context, widget) == GDK_NONE)
+    return FALSE;
+
   /* do nothing if the plugin is empty */
   if (plugin->items == NULL)
     {
@@ -1796,6 +1935,7 @@ launcher_plugin_button_drag_motion (GtkWidget      *widget,
   /* highlight the button if this is a launcher button */
   if (NO_ARROW_INSIDE_BUTTON (plugin))
     {
+      gdk_drag_status (context, GDK_ACTION_COPY, drag_time);
       gtk_drag_highlight (widget);
       return TRUE;
     }
@@ -1807,6 +1947,27 @@ launcher_plugin_button_drag_motion (GtkWidget      *widget,
 
 
 
+static gboolean
+launcher_plugin_button_drag_drop (GtkWidget      *widget,
+                                  GdkDragContext *context,
+                                  gint            x,
+                                  gint            y,
+                                  guint           drag_time,
+                                  LauncherPlugin *plugin)
+{
+  GdkAtom target;
+
+  target = launcher_plugin_supported_drop (context, widget);
+  if (target == GDK_NONE)
+    return FALSE;
+
+  gtk_drag_get_data (widget, context, target, drag_time);
+
+  return TRUE;
+}
+
+
+
 static void
 launcher_plugin_button_drag_leave (GtkWidget      *widget,
                                    GdkDragContext *context,
@@ -1929,6 +2090,9 @@ launcher_plugin_arrow_drag_motion (GtkWidget      *widget,
 {
   panel_return_val_if_fail (XFCE_IS_LAUNCHER_PLUGIN (plugin), FALSE);
 
+  if (launcher_plugin_supported_drop (context, widget) == GDK_NONE)
+    return FALSE;
+
   /* the arrow is not a drop zone */
   gdk_drag_status (context, 0, drag_time);
 
@@ -1942,11 +2106,9 @@ launcher_plugin_arrow_drag_motion (GtkWidget      *widget,
           g_timeout_add_full (G_PRIORITY_DEFAULT_IDLE, MENU_POPUP_DELAY,
                               launcher_plugin_menu_popup, plugin,
                               launcher_plugin_menu_popup_destroyed);
-
-      return TRUE;
     }
 
-  return FALSE;
+  return TRUE;
 }
 
 
@@ -2447,7 +2609,7 @@ launcher_plugin_garcon_menu_pool_add (GarconMenu *menu,
       if (!garcon_menu_element_get_visible (GARCON_MENU_ELEMENT (item)))
         continue;
 
-      /* skip dupplicates */
+      /* skip duplicates */
       desktop_id = garcon_menu_item_get_desktop_id (item);
       if (g_hash_table_lookup (pool, desktop_id) != NULL)
         continue;
