@@ -1,19 +1,19 @@
 /*
- * Copyright (c) 2010 Nick Schermer <nick@xfce.org>
+ * Copyright (C) 2010 Nick Schermer <nick@xfce.org>
  *
- * This program is free software; you can redistribute it and/or modify it
+ * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
  * Software Foundation; either version 2 of the License, or (at your option)
  * any later version.
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
+ * This library is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -280,14 +280,14 @@ pager_buttons_rebuild_idle (gpointer user_data)
   gint           rows, cols;
   gint           row, col;
   GtkWidget     *button;
-  WnckWorkspace *workspace;
+  WnckWorkspace *workspace = NULL;
   GtkWidget     *panel_plugin;
   GtkWidget     *label;
-  gint           workspace_width, workspace_height;
-  gint           screen_width, screen_height;
+  gint           workspace_width, workspace_height = 0;
+  gint           screen_width = 0, screen_height = 0;
   gint           viewport_x, viewport_y;
   gboolean       viewport_mode = FALSE;
-  gint           n_viewports;
+  gint           n_viewports = 0;
   gint          *vp_info;
   gchar          text[8];
 
@@ -335,10 +335,14 @@ pager_buttons_rebuild_idle (gpointer user_data)
         {
           g_warning ("only viewports with equally distributed screens are supported: %dx%d & %dx%d",
                      workspace_width, workspace_height, screen_width, screen_height);
+
+          goto workspace_layout;
         }
     }
   else
     {
+      workspace_layout:
+
       rows = CLAMP (1, pager->rows, n_workspaces);
       cols = n_workspaces / rows;
       if (cols * rows < n_workspaces)
