@@ -2,21 +2,21 @@
  * Copyright (c) 2002      Anders Carlsson <andersca@gnu.org>
  * Copyright (c) 2003-2006 Vincent Untz
  * Copyright (c) 2008      Red Hat, Inc.
- * Copyright (c) 2009      Nick Schermer <nick@xfce.org>
+ * Copyright (c) 2009-2010 Nick Schermer <nick@xfce.org>
  *
- * This program is free software; you can redistribute it and/or modify it
+ * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
  * Software Foundation; either version 2 of the License, or (at your option)
  * any later version.
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
+ * This library is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -52,12 +52,16 @@ struct _SystraySocket
   /* plug window */
   GdkNativeWindow window;
 
-  guint           is_composited : 1;
-  guint           parent_relative_bg : 1;
+  gchar           *name;
+
+  guint            is_composited : 1;
+  guint            parent_relative_bg : 1;
+  guint            hidden : 1;
 };
 
 
 
+static void     systray_socket_finalize      (GObject        *object);
 static void     systray_socket_realize       (GtkWidget      *widget);
 static void     systray_socket_size_allocate (GtkWidget      *widget,
                                               GtkAllocation  *allocation);
@@ -76,6 +80,10 @@ static void
 systray_socket_class_init (SystraySocketClass *klass)
 {
   GtkWidgetClass *gtkwidget_class;
+  GObjectClass   *gobject_class;
+
+  gobject_class = G_OBJECT_CLASS (klass);
+  gobject_class->finalize = systray_socket_finalize;
 
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
   gtkwidget_class->realize = systray_socket_realize;
@@ -89,6 +97,20 @@ systray_socket_class_init (SystraySocketClass *klass)
 static void
 systray_socket_init (SystraySocket *socket)
 {
+  socket->hidden = FALSE;
+  socket->name = NULL;
+}
+
+
+
+static void
+systray_socket_finalize (GObject *object)
+{
+  SystraySocket *socket = XFCE_SYSTRAY_SOCKET (object);
+
+  g_free (socket->name);
+
+  G_OBJECT_CLASS (systray_socket_parent_class)->finalize (object);
 }
 
 
@@ -307,50 +329,53 @@ systray_socket_is_composited (SystraySocket *socket)
 
 
 
-gchar *
-systray_socket_get_title  (SystraySocket *socket)
+static gchar *
+systray_socket_get_name_prop (SystraySocket *socket,
+                              const gchar   *prop_name,
+                              const gchar   *type_name)
 {
-  gchar      *name = NULL;
   GdkDisplay *display;
-  Atom        utf8_string, type;
+  Atom        req_type, type;
   gint        result;
+  gchar      *val;
   gint        format;
   gulong      nitems;
   gulong      bytes_after;
-  gchar      *val;
+  gchar      *name = NULL;
 
   panel_return_val_if_fail (XFCE_IS_SYSTRAY_SOCKET (socket), NULL);
+  panel_return_val_if_fail (type_name != NULL && prop_name != NULL, NULL);
 
   display = gtk_widget_get_display (GTK_WIDGET (socket));
 
-  utf8_string = gdk_x11_get_xatom_by_name_for_display (display, "UTF8_STRING");
+  req_type = gdk_x11_get_xatom_by_name_for_display (display, type_name);
 
   gdk_error_trap_push ();
 
   result = XGetWindowProperty (GDK_DISPLAY_XDISPLAY (display),
                                socket->window,
-                               gdk_x11_get_xatom_by_name_for_display (display, "_NET_WM_NAME"),
+                               gdk_x11_get_xatom_by_name_for_display (display, prop_name),
                                0, G_MAXLONG, False,
-                               utf8_string,
+                               req_type,
                                &type, &format, &nitems,
-                               &bytes_after, (guchar **) &val);
+                               &bytes_after,
+                               (guchar **) &val);
 
-  if (gdk_error_trap_pop () != 0 || result != Success || val == NULL)
+  /* check if everything went fine */
+  if (gdk_error_trap_pop () != 0
+      || result != Success
+      || val == NULL)
     return NULL;
 
-  if (type != utf8_string || format != 8 || nitems == 0)
-    {
-      XFree (val);
-      return NULL;
-    }
-
-  if (!g_utf8_validate (val, nitems, NULL))
-    {
-      XFree (val);
-      return NULL;
-    }
-
-  name = g_utf8_strdown (val, nitems);
+  /* check the returned data */
+  if (type == req_type
+      && format == 8
+      && nitems > 0
+      && g_utf8_validate (val, nitems, NULL))
+   {
+     /* lowercase the result */
+     name = g_utf8_strdown (val, nitems);
+   }
 
   XFree (val);
 
@@ -359,6 +384,25 @@ systray_socket_get_title  (SystraySocket *socket)
 
 
 
+const gchar *
+systray_socket_get_name  (SystraySocket *socket)
+{
+  panel_return_val_if_fail (XFCE_IS_SYSTRAY_SOCKET (socket), NULL);
+
+  if (G_LIKELY (socket->name != NULL))
+    return socket->name;
+
+  /* try _NET_WM_NAME first, for gtk icon implementations, fall back to
+   * WM_NAME for qt icons */
+  socket->name = systray_socket_get_name_prop (socket, "_NET_WM_NAME", "UTF8_STRING");
+  if (G_UNLIKELY (socket->name == NULL))
+    socket->name = systray_socket_get_name_prop (socket, "WM_NAME", "STRING");
+
+  return socket->name;
+}
+
+
+
 GdkNativeWindow *
 systray_socket_get_window (SystraySocket *socket)
 {
@@ -366,3 +410,24 @@ systray_socket_get_window (SystraySocket *socket)
 
   return &socket->window;
 }
+
+
+
+gboolean
+systray_socket_get_hidden (SystraySocket *socket)
+{
+  panel_return_val_if_fail (XFCE_IS_SYSTRAY_SOCKET (socket), FALSE);
+
+  return socket->hidden;
+}
+
+
+
+void
+systray_socket_set_hidden (SystraySocket *socket,
+                           gboolean       hidden)
+{
+  panel_return_if_fail (XFCE_IS_SYSTRAY_SOCKET (socket));
+
+  socket->hidden = hidden;
+}
