@@ -2,21 +2,21 @@
  * Copyright (c) 2002      Anders Carlsson <andersca@gnu.org>
  * Copyright (c) 2003-2006 Vincent Untz
  * Copyright (c) 2008      Red Hat, Inc.
- * Copyright (c) 2009      Nick Schermer <nick@xfce.org>
+ * Copyright (c) 2009-2010 Nick Schermer <nick@xfce.org>
  *
- * This program is free software; you can redistribute it and/or modify it
+ * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
  * Software Foundation; either version 2 of the License, or (at your option)
  * any later version.
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
+ * This library is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Library General Public License
+ * along with this library; if not, write to the Free Software Foundation,
+ * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #ifndef __SYSTRAY_SOCKET_H__
@@ -45,8 +45,13 @@ void             systray_socket_force_redraw  (SystraySocket   *socket);
 
 gboolean         systray_socket_is_composited (SystraySocket   *socket);
 
-gchar           *systray_socket_get_title     (SystraySocket   *socket) G_GNUC_MALLOC;
+const gchar     *systray_socket_get_name      (SystraySocket   *socket);
 
 GdkNativeWindow *systray_socket_get_window    (SystraySocket   *socket);
 
+gboolean         systray_socket_get_hidden    (SystraySocket   *socket);
+
+void             systray_socket_set_hidden    (SystraySocket   *socket,
+                                               gboolean         hidden);
+
 #endif /* !__SYSTRAY_SOCKET_H__ */
