@@ -34,8 +34,9 @@
 #include "systray-manager.h"
 #include "systray-dialog_ui.h"
 
-#define ICON_SIZE (22)
-
+#define ICON_SIZE     (22)
+#define BUTTON_SIZE   (16)
+#define FRAME_SPACING (1)
 
 
 static void     systray_plugin_get_property                 (GObject               *object,
@@ -48,13 +49,28 @@ static void     systray_plugin_set_property                 (GObject
                                                              GParamSpec            *pspec);
 static void     systray_plugin_construct                    (XfcePanelPlugin       *panel_plugin);
 static void     systray_plugin_free_data                    (XfcePanelPlugin       *panel_plugin);
-static void     systray_plugin_screen_position_changed      (XfcePanelPlugin       *panel_plugin,
-                                                             XfceScreenPosition     screen_position);
 static void     systray_plugin_orientation_changed          (XfcePanelPlugin       *panel_plugin,
                                                              GtkOrientation         orientation);
 static gboolean systray_plugin_size_changed                 (XfcePanelPlugin       *panel_plugin,
                                                              gint                   size);
 static void     systray_plugin_configure_plugin             (XfcePanelPlugin       *panel_plugin);
+static void     systray_plugin_box_expose_event             (GtkWidget             *box,
+                                                             GdkEventExpose        *event);
+static void     systray_plugin_button_toggled               (GtkWidget             *button,
+                                                             SystrayPlugin         *plugin);
+static void     systray_plugin_button_set_arrow             (SystrayPlugin         *plugin);
+static void     systray_plugin_names_collect_visible        (gpointer               key,
+                                                             gpointer               value,
+                                                             gpointer               user_data);
+static void     systray_plugin_names_collect_hidden         (gpointer               key,
+                                                             gpointer               value,
+                                                             gpointer               user_data);
+static gboolean systray_plugin_names_remove                 (gpointer               key,
+                                                             gpointer               value,
+                                                             gpointer               user_data);
+static void     systray_plugin_names_update                 (SystrayPlugin         *plugin);
+static gboolean systray_plugin_names_get_hidden             (SystrayPlugin         *plugin,
+                                                             const gchar           *name);
 static void     systray_plugin_icon_added                   (SystrayManager        *manager,
                                                              GtkWidget             *icon,
                                                              SystrayPlugin         *plugin);
@@ -63,8 +79,9 @@ static void     systray_plugin_icon_removed                 (SystrayManager
                                                              SystrayPlugin         *plugin);
 static void     systray_plugin_lost_selection               (SystrayManager        *manager,
                                                              SystrayPlugin         *plugin);
-static void     systray_plugin_dialog_add_application_names (SystrayPlugin         *plugin,
-                                                             GtkListStore          *store);
+static void     systray_plugin_dialog_add_application_names (gpointer               key,
+                                                             gpointer               value,
+                                                             gpointer               user_data);
 static void     systray_plugin_dialog_hidden_toggled        (GtkCellRendererToggle *renderer,
                                                              const gchar           *path_string,
                                                              SystrayPlugin         *plugin);
@@ -89,17 +106,22 @@ struct _SystrayPlugin
 
   /* widgets */
   GtkWidget      *frame;
+  GtkWidget      *hvbox;
   GtkWidget      *box;
+  GtkWidget      *button;
 
   /* settings */
   guint           show_frame : 1;
+  GHashTable     *names;
 };
 
 enum
 {
   PROP_0,
-  PROP_ROWS,
-  PROP_SHOW_FRAME
+  PROP_SIZE_MAX,
+  PROP_SHOW_FRAME,
+  PROP_NAMES_HIDDEN,
+  PROP_NAMES_VISIBLE
 };
 
 enum
@@ -126,6 +148,10 @@ static const gchar *known_applications[][3] =
   /* application name, icon-name, understandable name */
   { "networkmanager applet", "network-workgroup", "Network Manager Applet" },
   { "thunar", "Thunar", "Thunar Progress Dialog" },
+  { "workrave", NULL, "Workrave" },
+  { "workrave tray icon", NULL, "Workrave Applet" },
+  { "audacious2", "audacious", "Audacious" },
+  { "wicd-client.py", "wicd-gtk", "Wicd" },
 };
 
 
@@ -144,15 +170,16 @@ systray_plugin_class_init (SystrayPluginClass *klass)
   plugin_class->construct = systray_plugin_construct;
   plugin_class->free_data = systray_plugin_free_data;
   plugin_class->size_changed = systray_plugin_size_changed;
-  plugin_class->screen_position_changed = systray_plugin_screen_position_changed;
   plugin_class->configure_plugin = systray_plugin_configure_plugin;
   plugin_class->orientation_changed = systray_plugin_orientation_changed;
 
   g_object_class_install_property (gobject_class,
-                                   PROP_ROWS,
-                                   g_param_spec_uint ("rows",
+                                   PROP_SIZE_MAX,
+                                   g_param_spec_uint ("size-max",
                                                       NULL, NULL,
-                                                      1, 10, 1,
+                                                      SIZE_MAX_MIN,
+                                                      SIZE_MAX_MAX,
+                                                      SIZE_MAX_DEFAULT,
                                                       EXO_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
@@ -161,6 +188,20 @@ systray_plugin_class_init (SystrayPluginClass *klass)
                                                          NULL, NULL,
                                                          TRUE,
                                                          EXO_PARAM_READWRITE));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_NAMES_HIDDEN,
+                                   g_param_spec_boxed ("names-hidden",
+                                                       NULL, NULL,
+                                                       PANEL_PROPERTIES_TYPE_VALUE_ARRAY,
+                                                       EXO_PARAM_READWRITE));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_NAMES_VISIBLE,
+                                   g_param_spec_boxed ("names-visible",
+                                                       NULL, NULL,
+                                                       PANEL_PROPERTIES_TYPE_VALUE_ARRAY,
+                                                       EXO_PARAM_READWRITE));
 }
 
 
@@ -171,16 +212,31 @@ systray_plugin_init (SystrayPlugin *plugin)
   plugin->manager = NULL;
   plugin->show_frame = TRUE;
   plugin->idle_startup = 0;
+  plugin->names = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
   plugin->frame = gtk_frame_new (NULL);
   gtk_container_add (GTK_CONTAINER (plugin), plugin->frame);
   gtk_frame_set_shadow_type (GTK_FRAME (plugin->frame), GTK_SHADOW_ETCHED_IN);
   gtk_widget_show (plugin->frame);
 
+  plugin->hvbox = xfce_hvbox_new (GTK_ORIENTATION_HORIZONTAL, FALSE, 2);
+  gtk_container_add (GTK_CONTAINER (plugin->frame), plugin->hvbox);
+  gtk_widget_show (plugin->hvbox);
+
   plugin->box = systray_box_new ();
-  gtk_container_add (GTK_CONTAINER (plugin->frame), plugin->box);
-  xfce_panel_plugin_add_action_widget (XFCE_PANEL_PLUGIN (plugin), plugin->box);
+  gtk_box_pack_start (GTK_BOX (plugin->hvbox), plugin->box, TRUE, TRUE, 0);
+  g_signal_connect (G_OBJECT (plugin->box), "expose-event",
+      G_CALLBACK (systray_plugin_box_expose_event), NULL);
+  gtk_container_set_border_width (GTK_CONTAINER (plugin->box), FRAME_SPACING);
   gtk_widget_show (plugin->box);
+
+  plugin->button = xfce_arrow_button_new (GTK_ARROW_RIGHT);
+  gtk_box_pack_start (GTK_BOX (plugin->hvbox), plugin->button, FALSE, FALSE, 0);
+  g_signal_connect (G_OBJECT (plugin->button), "toggled",
+      G_CALLBACK (systray_plugin_button_toggled), plugin);
+  gtk_button_set_relief (GTK_BUTTON (plugin->button), GTK_RELIEF_NONE);
+  exo_binding_new (G_OBJECT (plugin->box), "has-hidden", G_OBJECT (plugin->button), "visible");
+  xfce_panel_plugin_add_action_widget (XFCE_PANEL_PLUGIN (plugin), plugin->button);
 }
 
 
@@ -192,19 +248,33 @@ systray_plugin_get_property (GObject    *object,
                              GParamSpec *pspec)
 {
   SystrayPlugin *plugin = XFCE_SYSTRAY_PLUGIN (object);
-  guint          rows;
+  GPtrArray     *array;
 
   switch (prop_id)
     {
-    case PROP_ROWS:
-      rows = systray_box_get_rows (XFCE_SYSTRAY_BOX (plugin->box));
-      g_value_set_uint (value, rows);
+    case PROP_SIZE_MAX:
+      g_value_set_uint (value,
+          systray_box_get_size_max (XFCE_SYSTRAY_BOX (plugin->box)));
       break;
 
     case PROP_SHOW_FRAME:
       g_value_set_boolean (value, plugin->show_frame);
       break;
 
+   case PROP_NAMES_VISIBLE:
+      array = g_ptr_array_new ();
+      g_hash_table_foreach (plugin->names, systray_plugin_names_collect_visible, array);
+      g_value_set_boxed (value, array);
+      xfconf_array_free (array);
+      break;
+
+    case PROP_NAMES_HIDDEN:
+      array = g_ptr_array_new ();
+      g_hash_table_foreach (plugin->names, systray_plugin_names_collect_hidden, array);
+      g_value_set_boxed (value, array);
+      xfconf_array_free (array);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -221,12 +291,17 @@ systray_plugin_set_property (GObject      *object,
 {
   SystrayPlugin *plugin = XFCE_SYSTRAY_PLUGIN (object);
   gboolean       show_frame;
+  gboolean       hidden = TRUE;
+  GPtrArray     *array;
+  const GValue  *tmp;
+  gchar         *name;
+  guint          i;
 
   switch (prop_id)
     {
-    case PROP_ROWS:
-      systray_box_set_rows (XFCE_SYSTRAY_BOX (plugin->box),
-                            g_value_get_uint (value));
+    case PROP_SIZE_MAX:
+      systray_box_set_size_max (XFCE_SYSTRAY_BOX (plugin->box),
+                                g_value_get_uint (value));
       break;
 
     case PROP_SHOW_FRAME:
@@ -236,9 +311,39 @@ systray_plugin_set_property (GObject      *object,
           plugin->show_frame = show_frame;
           gtk_frame_set_shadow_type (GTK_FRAME (plugin->frame),
               show_frame ? GTK_SHADOW_ETCHED_IN : GTK_SHADOW_NONE);
+
+          gtk_container_set_border_width (GTK_CONTAINER (plugin->box),
+              plugin->show_frame ? FRAME_SPACING : 0);
         }
       break;
 
+    case PROP_NAMES_VISIBLE:
+      hidden = FALSE;
+      /* fall-though */
+
+    case PROP_NAMES_HIDDEN:
+      /* remove old names with this state */
+      g_hash_table_foreach_remove (plugin->names,
+                                   systray_plugin_names_remove,
+                                   GUINT_TO_POINTER (hidden));
+
+      /* add new values */
+      array = g_value_get_boxed (value);
+      if (G_LIKELY (array != NULL))
+        {
+          for (i = 0; i < array->len; i++)
+            {
+              tmp = g_ptr_array_index (array, i);
+              panel_assert (G_VALUE_HOLDS_STRING (tmp));
+              name = g_value_dup_string (tmp);
+              g_hash_table_replace (plugin->names, name, GUINT_TO_POINTER (hidden));
+            }
+        }
+
+      /* update icons in the box */
+      systray_plugin_names_update (plugin);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -318,12 +423,8 @@ systray_plugin_construct (XfcePanelPlugin *panel_plugin)
   SystrayPlugin       *plugin = XFCE_SYSTRAY_PLUGIN (panel_plugin);
   const PanelProperty  properties[] =
   {
-    { "rows", G_TYPE_UINT },
+    { "size-max", G_TYPE_UINT },
     { "show-frame", G_TYPE_BOOLEAN },
-    { NULL }
-  };
-  const PanelProperty  box_properties[] =
-  {
     { "names-visible", PANEL_PROPERTIES_TYPE_VALUE_ARRAY },
     { "names-hidden", PANEL_PROPERTIES_TYPE_VALUE_ARRAY },
     { NULL }
@@ -331,13 +432,9 @@ systray_plugin_construct (XfcePanelPlugin *panel_plugin)
 
   xfce_panel_plugin_menu_show_configure (XFCE_PANEL_PLUGIN (plugin));
 
-  /* bind all properties */
   panel_properties_bind (NULL, G_OBJECT (plugin),
                          xfce_panel_plugin_get_property_base (panel_plugin),
                          properties, FALSE);
-  panel_properties_bind (NULL, G_OBJECT (plugin->box),
-                         xfce_panel_plugin_get_property_base (panel_plugin),
-                         box_properties, FALSE);
 
   /* monitor screen changes */
   g_signal_connect (G_OBJECT (plugin), "screen-changed",
@@ -360,6 +457,8 @@ systray_plugin_free_data (XfcePanelPlugin *panel_plugin)
   g_signal_handlers_disconnect_by_func (G_OBJECT (plugin),
       systray_plugin_screen_changed, NULL);
 
+  g_hash_table_destroy (plugin->names);
+
   if (G_LIKELY (plugin->manager != NULL))
     {
       systray_manager_unregister (plugin->manager);
@@ -369,84 +468,24 @@ systray_plugin_free_data (XfcePanelPlugin *panel_plugin)
 
 
 
-static void
-systray_plugin_screen_position_changed (XfcePanelPlugin    *panel_plugin,
-                                        XfceScreenPosition  screen_position)
-{
-  SystrayPlugin      *plugin = XFCE_SYSTRAY_PLUGIN (panel_plugin);
-  GdkScreen          *screen;
-  GdkRectangle        geom;
-  gint                mon, x, y;
-  GtkArrowType        arrow_type;
-
-  panel_return_if_fail (GTK_WIDGET_REALIZED (panel_plugin));
-
-  /* get the button position */
-  switch (screen_position)
-    {
-    /*    horizontal west */
-    case XFCE_SCREEN_POSITION_NW_H:
-    case XFCE_SCREEN_POSITION_SW_H:
-      arrow_type = GTK_ARROW_RIGHT;
-      break;
-
-    /* horizontal east */
-    case XFCE_SCREEN_POSITION_N:
-    case XFCE_SCREEN_POSITION_NE_H:
-    case XFCE_SCREEN_POSITION_S:
-    case XFCE_SCREEN_POSITION_SE_H:
-      arrow_type = GTK_ARROW_LEFT;
-      break;
-
-    /* vertical north */
-    case XFCE_SCREEN_POSITION_NW_V:
-    case XFCE_SCREEN_POSITION_NE_V:
-      arrow_type = GTK_ARROW_DOWN;
-      break;
-
-    /* vertical south */
-    case XFCE_SCREEN_POSITION_W:
-    case XFCE_SCREEN_POSITION_SW_V:
-    case XFCE_SCREEN_POSITION_E:
-    case XFCE_SCREEN_POSITION_SE_V:
-      arrow_type = GTK_ARROW_UP;
-      break;
-
-    /* floating */
-    default:
-      /* get the screen information */
-      screen = gtk_widget_get_screen (GTK_WIDGET (panel_plugin));
-      mon = gdk_screen_get_monitor_at_window (screen, GTK_WIDGET (panel_plugin)->window);
-      gdk_screen_get_monitor_geometry (screen, mon, &geom);
-      gdk_window_get_root_origin (GTK_WIDGET (panel_plugin)->window, &x, &y);
-
-      /* get the position based on the screen position */
-      if (screen_position == XFCE_SCREEN_POSITION_FLOATING_H)
-          arrow_type = ((x < (geom.x + geom.width / 2)) ? GTK_ARROW_RIGHT : GTK_ARROW_LEFT);
-      else
-          arrow_type = ((y < (geom.y + geom.height / 2)) ? GTK_ARROW_DOWN : GTK_ARROW_UP);
-      break;
-    }
-
-  /* set the arrow type of the tray widget */
-  systray_box_set_arrow_type (XFCE_SYSTRAY_BOX (plugin->box), arrow_type);
-
-  /* update the manager orientation */
-  systray_plugin_orientation_changed (panel_plugin,
-      xfce_panel_plugin_get_orientation (panel_plugin));
-}
-
-
-
 static void
 systray_plugin_orientation_changed (XfcePanelPlugin *panel_plugin,
                                     GtkOrientation   orientation)
 {
   SystrayPlugin *plugin = XFCE_SYSTRAY_PLUGIN (panel_plugin);
 
-  /* send the new orientation to the manager */
+  xfce_hvbox_set_orientation (XFCE_HVBOX (plugin->hvbox), orientation);
+  systray_box_set_orientation (XFCE_SYSTRAY_BOX (plugin->box), orientation);
+
   if (G_LIKELY (plugin->manager != NULL))
     systray_manager_set_orientation (plugin->manager, orientation);
+
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    gtk_widget_set_size_request (plugin->button, BUTTON_SIZE, -1);
+  else
+    gtk_widget_set_size_request (plugin->button, -1, BUTTON_SIZE);
+
+  systray_plugin_button_set_arrow (plugin);
 }
 
 
@@ -464,10 +503,12 @@ systray_plugin_size_changed (XfcePanelPlugin *panel_plugin,
     border = 1;
   gtk_container_set_border_width (GTK_CONTAINER (frame), border);
 
-  /* set the guess size this is used to get the initial icon size request
-   * correct to avoid flickering in the system tray for new applications */
+  /* because the allocated size, used in size_requested is always 1 step
+   * behind the allocated size when resizing and during startup, we
+   * correct the maximum size set by the user with the size the panel
+   * will most likely allocated */
   border += MAX (frame->style->xthickness, frame->style->ythickness);
-  systray_box_set_guess_size (XFCE_SYSTRAY_BOX (plugin->box), size - 2 * border);
+  systray_box_set_size_alloc (XFCE_SYSTRAY_BOX (plugin->box), size - 2 * border);
 
   return TRUE;
 }
@@ -488,9 +529,9 @@ systray_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
   if (G_UNLIKELY (builder == NULL))
     return;
 
-  object = gtk_builder_get_object (builder, "rows");
+  object = gtk_builder_get_object (builder, "size-max");
   panel_return_if_fail (GTK_IS_WIDGET (object));
-  exo_mutual_binding_new (G_OBJECT (plugin), "rows",
+  exo_mutual_binding_new (G_OBJECT (plugin), "size-max",
                           G_OBJECT (object), "value");
 
   object = gtk_builder_get_object (builder, "show-frame");
@@ -500,7 +541,8 @@ systray_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
 
   store = gtk_builder_get_object (builder, "applications-store");
   panel_return_if_fail (GTK_IS_LIST_STORE (store));
-  systray_plugin_dialog_add_application_names (plugin, GTK_LIST_STORE (store));
+  g_hash_table_foreach (plugin->names,
+      systray_plugin_dialog_add_application_names, store);
   g_object_set_data (G_OBJECT (plugin), "applications-store", store);
 
   object = gtk_builder_get_object (builder, "hidden-toggle");
@@ -518,22 +560,242 @@ systray_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
 
 
 
+static void
+systray_plugin_box_expose_event_icon (GtkWidget *child,
+                                      gpointer   user_data)
+{
+  cairo_t       *cr = user_data;
+  GtkAllocation *alloc;
+
+  if (systray_socket_is_composited (XFCE_SYSTRAY_SOCKET (child)))
+    {
+      alloc = &child->allocation;
+
+      /* skip hidden (see offscreen in box widget) icons */
+      if (alloc->x > 0 && alloc->y > 0)
+        {
+          gdk_cairo_set_source_pixmap (cr, gtk_widget_get_window (child),
+                                       alloc->x, alloc->y);
+          cairo_paint (cr);
+        }
+    }
+}
+
+
+
+static void
+systray_plugin_box_expose_event (GtkWidget      *box,
+                                 GdkEventExpose *event)
+{
+  cairo_t *cr;
+
+  if (!gtk_widget_is_composited (box))
+    return;
+
+  cr = gdk_cairo_create (gtk_widget_get_window (box));
+  if (G_LIKELY (cr != NULL))
+    {
+      gdk_cairo_rectangle (cr, &event->area);
+      cairo_clip (cr);
+
+      /* separately draw all the composed tray icons after gtk
+       * handled the expose event */
+      gtk_container_foreach (GTK_CONTAINER (box),
+          systray_plugin_box_expose_event_icon, cr);
+
+      cairo_destroy (cr);
+    }
+}
+
+
+
+static void
+systray_plugin_button_toggled (GtkWidget     *button,
+                               SystrayPlugin *plugin)
+{
+  panel_return_if_fail (XFCE_IS_SYSTRAY_PLUGIN (plugin));
+  panel_return_if_fail (GTK_IS_TOGGLE_BUTTON (button));
+  panel_return_if_fail (plugin->button == button);
+
+  systray_box_set_show_hidden (XFCE_SYSTRAY_BOX (plugin->box),
+      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)));
+  systray_plugin_button_set_arrow (plugin);
+}
+
+
+
+static void
+systray_plugin_button_set_arrow (SystrayPlugin *plugin)
+{
+  GtkArrowType   arrow_type;
+  gboolean       show_hidden;
+  GtkOrientation orientation;
+
+  panel_return_if_fail (XFCE_IS_SYSTRAY_PLUGIN (plugin));
+
+  show_hidden = systray_box_get_show_hidden (XFCE_SYSTRAY_BOX (plugin->box));
+  orientation = xfce_panel_plugin_get_orientation (XFCE_PANEL_PLUGIN (plugin));
+  if (orientation == GTK_ORIENTATION_HORIZONTAL)
+    arrow_type = show_hidden ? GTK_ARROW_LEFT : GTK_ARROW_RIGHT;
+  else
+    arrow_type = show_hidden ? GTK_ARROW_UP : GTK_ARROW_DOWN;
+
+  xfce_arrow_button_set_arrow_type (XFCE_ARROW_BUTTON (plugin->button), arrow_type);
+}
+
+
+
+static void
+systray_plugin_names_collect (GPtrArray   *array,
+                              const gchar *name)
+{
+  GValue *tmp;
+
+  tmp = g_new0 (GValue, 1);
+  g_value_init (tmp, G_TYPE_STRING);
+  g_value_set_string (tmp, name);
+  g_ptr_array_add (array, tmp);
+}
+
+
+
+static void
+systray_plugin_names_collect_visible (gpointer key,
+                                      gpointer value,
+                                      gpointer user_data)
+{
+  /* add all the visible names */
+  if (!GPOINTER_TO_UINT (value))
+    systray_plugin_names_collect (user_data, key);
+}
+
+
+
+static void
+systray_plugin_names_collect_hidden (gpointer key,
+                                     gpointer value,
+                                     gpointer user_data)
+{
+  /* add all the hidden names */
+  if (GPOINTER_TO_UINT (value))
+    systray_plugin_names_collect (user_data, key);
+}
+
+
+
+static gboolean
+systray_plugin_names_remove (gpointer key,
+                             gpointer value,
+                             gpointer user_data)
+{
+  return GPOINTER_TO_UINT (value) == GPOINTER_TO_UINT (user_data);
+}
+
+
+
+static void
+systray_plugin_names_update_icon (GtkWidget *icon,
+                                  gpointer   data)
+{
+  SystrayPlugin *plugin = XFCE_SYSTRAY_PLUGIN (data);
+  SystraySocket *socket = XFCE_SYSTRAY_SOCKET (icon);
+  const gchar   *name;
+
+  panel_return_if_fail (XFCE_IS_SYSTRAY_PLUGIN (plugin));
+  panel_return_if_fail (XFCE_IS_SYSTRAY_SOCKET (icon));
+
+  name = systray_socket_get_name (socket);
+  systray_socket_set_hidden (socket,
+      systray_plugin_names_get_hidden (plugin, name));
+}
+
+
+
+static void
+systray_plugin_names_update (SystrayPlugin *plugin)
+{
+  panel_return_if_fail (XFCE_IS_SYSTRAY_PLUGIN (plugin));
+
+  gtk_container_foreach (GTK_CONTAINER (plugin->box),
+     systray_plugin_names_update_icon, plugin);
+  systray_box_update (XFCE_SYSTRAY_BOX (plugin->box));
+}
+
+
+
+static void
+systray_plugin_names_set_hidden (SystrayPlugin *plugin,
+                                 const gchar   *name,
+                                 gboolean       hidden)
+{
+  panel_return_if_fail (XFCE_IS_SYSTRAY_PLUGIN (plugin));
+  panel_return_if_fail (!exo_str_is_empty (name));
+
+  g_hash_table_replace (plugin->names, g_strdup (name),
+                        GUINT_TO_POINTER (hidden ? 1 : 0));
+
+  systray_plugin_names_update (plugin);
+
+  g_object_notify (G_OBJECT (plugin), "names-visible");
+  g_object_notify (G_OBJECT (plugin), "names-hidden");
+}
+
+
+
+static gboolean
+systray_plugin_names_get_hidden (SystrayPlugin *plugin,
+                                 const gchar   *name)
+{
+  gpointer p;
+
+  if (exo_str_is_empty (name))
+    return FALSE;
+
+  /* lookup the name in the table */
+  p = g_hash_table_lookup (plugin->names, name);
+  if (G_UNLIKELY (p == NULL))
+    {
+      /* add the new name */
+      g_hash_table_insert (plugin->names, g_strdup (name), GUINT_TO_POINTER (0));
+      g_object_notify (G_OBJECT (plugin), "names-visible");
+
+      /* do not hide the icon */
+      return FALSE;
+    }
+  else
+    {
+      return (GPOINTER_TO_UINT (p) == 1 ? TRUE : FALSE);
+    }
+}
+
+
+
+static void
+systray_plugin_names_clear (SystrayPlugin *plugin)
+{
+  g_hash_table_remove_all (plugin->names);
+
+  g_object_notify (G_OBJECT (plugin), "names-hidden");
+  g_object_notify (G_OBJECT (plugin), "names-visible");
+
+  systray_plugin_names_update (plugin);
+}
+
+
+
 static void
 systray_plugin_icon_added (SystrayManager *manager,
                            GtkWidget      *icon,
                            SystrayPlugin  *plugin)
 {
-  gchar *name;
-
   panel_return_if_fail (XFCE_IS_SYSTRAY_MANAGER (manager));
   panel_return_if_fail (XFCE_IS_SYSTRAY_PLUGIN (plugin));
+  panel_return_if_fail (XFCE_IS_SYSTRAY_SOCKET (icon));
   panel_return_if_fail (plugin->manager == manager);
   panel_return_if_fail (GTK_IS_WIDGET (icon));
 
-  name = systray_socket_get_title (XFCE_SYSTRAY_SOCKET (icon));
-  systray_box_add_with_name (XFCE_SYSTRAY_BOX (plugin->box), icon, name);
-  g_free (name);
-
+  systray_plugin_names_update_icon (icon, plugin);
+  gtk_container_add (GTK_CONTAINER (plugin->box), icon);
   gtk_widget_show (icon);
 }
 
@@ -614,107 +876,64 @@ systray_plugin_dialog_camel_case (const gchar *text)
 
 
 
-static GdkPixbuf *
-systray_plugin_dialog_icon (GtkIconTheme *icon_theme,
-                            const gchar  *icon_name)
-{
-  GdkPixbuf   *icon = NULL;
-  gchar       *first_occ;
-  const gchar *p;
-
-  panel_return_val_if_fail (!exo_str_is_empty (icon_name), NULL);
-  panel_return_val_if_fail (GTK_IS_ICON_THEME (icon_theme), NULL);
-
-  /* try to load the icon from the theme */
-  icon = gtk_icon_theme_load_icon (icon_theme, icon_name, ICON_SIZE, 0, NULL);
-  if (icon != NULL)
-    return icon;
-
-  /* try the first part when the name contains a space */
-  p = g_utf8_strchr (icon_name, -1, ' ');
-  if (p != NULL)
-    {
-      first_occ = g_strndup (icon_name, p - icon_name);
-      icon = gtk_icon_theme_load_icon (icon_theme, first_occ, ICON_SIZE, 0, NULL);
-      g_free (first_occ);
-    }
-
-  return icon;
-}
-
-
-
 static void
-systray_plugin_dialog_add_application_names (SystrayPlugin *plugin,
-                                             GtkListStore  *store)
+systray_plugin_dialog_add_application_names (gpointer key,
+                                             gpointer value,
+                                             gpointer user_data)
 {
-  GList        *names, *li;
-  const gchar  *name;
+  GtkListStore *store = GTK_LIST_STORE (user_data);
+  const gchar  *name = key;
+  gboolean      hidden = GPOINTER_TO_UINT (value);
+  const gchar  *title = NULL;
+  gchar        *camelcase = NULL;
+  const gchar  *icon_name = name;
+  GdkPixbuf    *pixbuf;
   guint         i;
-  gboolean      hidden;
-  const gchar  *title, *icon_name;
-  gchar        *camelcase;
-  GdkPixbuf    *icon;
-  GtkIconTheme *icon_theme;
   GtkTreeIter   iter;
 
-  panel_return_if_fail (XFCE_IS_SYSTRAY_PLUGIN (plugin));
-  panel_return_if_fail (XFCE_IS_SYSTRAY_BOX (plugin->box));
   panel_return_if_fail (GTK_IS_LIST_STORE (store));
+  panel_return_if_fail (name == NULL || g_utf8_validate (name, -1, NULL));
 
-  icon_theme = gtk_icon_theme_get_default ();
+  /* skip invalid names */
+  if (exo_str_is_empty (name))
+     return;
 
-  /* get the know application names and insert them in the store */
-  names = systray_box_name_list (XFCE_SYSTRAY_BOX (plugin->box));
-  for (li = names; li != NULL; li = li->next)
+  /* check if we have a better name for the application */
+  for (i = 0; i < G_N_ELEMENTS (known_applications); i++)
     {
-      name = li->data;
-
-      /* skip invalid names */
-      if (exo_str_is_empty (name))
-        continue;
-
-      title = NULL;
-      icon_name = name;
-      camelcase = NULL;
-      hidden = systray_box_name_get_hidden (XFCE_SYSTRAY_BOX (plugin->box), name);
-
-      /* check if we have a better name for the application */
-      for (i = 0; i < G_N_ELEMENTS (known_applications); i++)
+      if (strcmp (name, known_applications[i][0]) == 0)
         {
-          if (strcmp (name, known_applications[i][0]) == 0)
-            {
-              icon_name = known_applications[i][1];
-              title = _(known_applications[i][2]);
-              break;
-            }
+          icon_name = known_applications[i][1];
+          title = known_applications[i][2];
+          break;
         }
-
-      /* create fallback title if the application was not found */
-      if (title == NULL)
-        {
-          camelcase = systray_plugin_dialog_camel_case (name);
-          title = camelcase;
-        }
-
-      /* try to load the icon name */
-      icon = systray_plugin_dialog_icon (icon_theme, icon_name);
-
-      /* insert in the store */
-      gtk_list_store_append (store, &iter);
-      gtk_list_store_set (store, &iter,
-                          COLUMN_PIXBUF, icon,
-                          COLUMN_TITLE, title,
-                          COLUMN_HIDDEN, hidden,
-                          COLUMN_INTERNAL_NAME, name,
-                          -1);
-
-      g_free (camelcase);
-      if (icon != NULL)
-        g_object_unref (G_OBJECT (icon));
     }
 
-  g_list_free (names);
+  /* create fallback title if the application was not found */
+  if (title == NULL)
+    {
+      camelcase = systray_plugin_dialog_camel_case (name);
+      title = camelcase;
+    }
+
+  /* try to load the icon name */
+  if (G_LIKELY (icon_name != NULL))
+    pixbuf = xfce_panel_pixbuf_from_source (icon_name, NULL, ICON_SIZE);
+  else
+    pixbuf = NULL;
+
+  /* insert in the store */
+  gtk_list_store_append (store, &iter);
+  gtk_list_store_set (store, &iter,
+                      COLUMN_PIXBUF, pixbuf,
+                      COLUMN_TITLE, title,
+                      COLUMN_HIDDEN, hidden,
+                      COLUMN_INTERNAL_NAME, name,
+                      -1);
+
+  g_free (camelcase);
+  if (pixbuf != NULL)
+    g_object_unref (G_OBJECT (pixbuf));
 }
 
 
@@ -740,9 +959,12 @@ systray_plugin_dialog_hidden_toggled (GtkCellRendererToggle *renderer,
                           COLUMN_HIDDEN, &hidden,
                           COLUMN_INTERNAL_NAME, &name, -1);
 
+      /* insert value (we need to update it) */
+      hidden = !hidden;
+
       /* update box and store with new state */
-      systray_box_name_set_hidden (XFCE_SYSTRAY_BOX (plugin->box), name, !hidden);
-      gtk_list_store_set (GTK_LIST_STORE (model), &iter, 2, !hidden, -1);
+      systray_plugin_names_set_hidden (plugin, name, hidden);
+      gtk_list_store_set (GTK_LIST_STORE (model), &iter, 2, hidden, -1);
 
       g_free (name);
     }
@@ -768,6 +990,6 @@ systray_plugin_dialog_clear_clicked (GtkWidget     *button,
       panel_return_if_fail (GTK_IS_LIST_STORE (store));
       gtk_list_store_clear (store);
 
-      systray_box_name_clear (XFCE_SYSTRAY_BOX (plugin->box));
+      systray_plugin_names_clear (plugin);
     }
 }
